//--------------------------------------------------------------
// Fichero:  pedroreina.js
// Objetivo: Programación de un tema para lightdm-webkit-greeter
// Fecha:    L.7.1.2019
// Autor:    Pedro Reina <pedro@pedroreina.net>
// Licencia: CC0 1.0 Universal
//           http://creativecommons.org/publicdomain/zero/1.0/
//--------------------------------------------------------------

//----------------------------
// Variables globales
//----------------------------

// Controles HTML
var BotonBorrar = null;
var BotonEntrar = null;
var BotonReiniciar = null;
var BotonApagar = null;
var BotonCerrar = null;
var InputUsuario = null;
var InputContrasena = null;
var EtiquetaHost = null;
var EtiquetaFecha = null;
var EtiquetaHora = null;
var DivError = null;

//------------------------------------------
// Programa principal: cuando se cargue la
// página completa, ejecutamos una función
//------------------------------------------

AgregaEvento(window, 'load', PreparaTodo, false);

//------------------------------------------
// Funciones para preparar el sistema
//------------------------------------------

//--------------------------------------------------------------
// La función que prepara todo el documento
function PreparaTodo()
  {
  ObtenIdentificadores();
  AgregaEnganches();
  RellenaDatos();
  }

//--------------------------------------------------------------
// Obtener los identificadores de los controles HTML
function ObtenIdentificadores()
  {
  // Averiguamos los id de los botones
  BotonBorrar = document.getElementById('Borrar');
  BotonEntrar = document.getElementById('Entrar');
  BotonReiniciar = document.getElementById('Reiniciar');
  BotonApagar = document.getElementById('Apagar');
  BotonCerrar = document.getElementById('Cerrar');
  InputUsuario = document.getElementById('usuario');
  InputContrasena = document.getElementById('contrasena');
  EtiquetaHost = document.getElementById('host');
  EtiquetaFecha = document.getElementById('fecha');
  EtiquetaHora = document.getElementById('hora');
  DivError = document.getElementById('error');
  }

//--------------------------------------------------------------
// La función que añade todos los "listeners" necesarios
function AgregaEnganches()
  {
  AgregaEvento(BotonBorrar, 'click', Borrar, false);
  AgregaEvento(BotonEntrar, 'click', Entrar, false);
  AgregaEvento(BotonReiniciar, 'click', Reiniciar, false);
  AgregaEvento(BotonApagar, 'click', Apagar, false);
  AgregaEvento(BotonCerrar, 'click', Cerrar, false);
  AgregaEvento(BotonCerrar, 'keyup', PasaCerrar, false);
  AgregaEvento(InputUsuario, 'keyup', FocoContrasena, false);
  AgregaEvento(InputContrasena, 'keyup', PasaEntrar, false);
  }

//------------------------------------------
// Funciones particulares
//------------------------------------------

//------------------------------------------
// Devuelve la fecha en mi formato: S.dD.mM.AAAA
function Fecha()
  {
  var LetraDia = 'DLMXJVS';
  var Hoy = new Date();
  var DiaSem = Hoy.getDay();
  var Dia = Hoy.getDate();
  var Mes = Hoy.getMonth()+1;
  var Ano = Hoy.getFullYear();
  return LetraDia.charAt(DiaSem) + '.' + Dia + '.' + Mes + '.' + Ano;
  }

//------------------------------------------
// Devuelve la hora en formato HH:MM:SS
function Hora()
  {
  var Hoy = new Date();
  var LaHora = DosDigitos(Hoy.getHours());
  var Minuto = DosDigitos(Hoy.getMinutes());
  var Segundo = DosDigitos(Hoy.getSeconds());
  return LaHora + ':' + Minuto + ':' + Segundo;
  }

//------------------------------------------
// Se asegura de que el número de entrada salga con dos dígitos
function DosDigitos(n)
  {
  var r = n;
  if (n<=10) { r = ('0'+n).slice(-2); }
  return r;
  }

//------------------------------------------
// Rellena los datos de la parte superior
// y lanza el reloj
function RellenaDatos()
  {
  EtiquetaFecha.innerHTML = Fecha();
  EtiquetaHost.innerHTML = lightdm.hostname;
  EtiquetaHora.innerHTML = Hora();

  setInterval(ActualizaDatos, 1000)
  }

// Actualiza la fecha y la hora
function ActualizaDatos()
  {
  EtiquetaFecha.innerHTML = Fecha();
  EtiquetaHora.innerHTML = Hora();
  }

//------------------------------------------
// Borra usuario y contraseña
function Borrar()
  {
  InputUsuario.value = '';
  InputContrasena.value = '';
  InputUsuario.focus();
  }

//------------------------------------------
// Aparece el DIV de error
function ActivarError()
  {
  DivError.style.zIndex = "2";
  BotonCerrar.focus();
  }

//------------------------------------------
// Desaparece el DIV de error
function Cerrar()
  {
  DivError.style.zIndex = "1";
  Borrar();
  BotonEntrar.value = 'Entrar';
  BotonEntrar.style.padding = '5px 7px 5px 28px';
  BotonEntrar.style.background = 'url(entrar.png) no-repeat 5px 50%';
  }

//------------------------------------------
// Si se pulsa ENTER, pasa el foco a la contraseña
function FocoContrasena(evento)
  {
  var Tecla = evento.keyCode;
  if (Tecla == 13)
    { InputContrasena.focus(); }
  }

//------------------------------------------
// Si se pulsa ENTER, se intenta la autentificación
function PasaEntrar(evento)
  {
  var Tecla = evento.keyCode;
  if (Tecla == 13)
    { Entrar(); }
  }

//------------------------------------------
// Si se pulsa ESPACIO o ESC, se elimina el DIV de error
function PasaCerrar(evento)
  {
  var Tecla = evento.keyCode;
  if (Tecla == 32 || Tecla == 27)
    { Cerrar(); }
  }

//------------------------------------------
// Enviamos la contraseña a LightDM
function Entrar()
  {
  BotonEntrar.style.padding = '5px 7px 5px 7px';
  BotonEntrar.style.background = 'none';
  BotonEntrar.value = 'Comprobando...';
  submitPassword();
  }

//------------------------------------------
// Pedimos a LightDM que reinicie el sistema
function Reiniciar()
  { lightdm.restart(); }

//------------------------------------------
// Pedimos a LightDM que apague el sistema
function Apagar()
  { lightdm.shutdown(); }

//------------------------------------------
// Funciones propias de LightDM
//------------------------------------------

//------------------------------------------
// La respuesta de LightDM a nuestro usuario y contraseña
function authentication_complete()
  {
  console.log("Autentificación completa");
  Usuario = lightdm.authentication_user;
  if (lightdm.is_authenticated)
    {
    console.log("El usuario " + Usuario + " se ha autentificado");
    lightdm.login(Usuario, lightdm.default_session);
    }
  else
    {
    console.log("error: el usuario " + Usuario + " no se ha autentificado");
    ActivarError();
    }
  }

//------------------------------------------
// Comenzamos la autentificación de un usuario
function submitPassword()
   {
   console.log("Petición de contraseña");
   lightdm.cancel_timed_login();
   lightdm.start_authentication(InputUsuario.value);
   }

//------------------------------------------
// Mandamos un texto al log de LightDM
function show_error(message)
   { console.log("error: " + message); }

//------------------------------------------
// LightDM nos pide la contraseña
function show_prompt(message)
   {
   console.log("prompt: " + message);
   lightdm.provide_secret(InputContrasena.value);
   }

//----------------------------
// Manejo de eventos
//----------------------------

// Técnicas explicadas en http://www.sitepoint.com/books/dhtml1/

//--------------------------------------------------------------
// Función portable que añade un evento
function AgregaEvento (elemento, evType, funcion, useCapture)
  {
  // Manejador de eventos compatible con navegadores IE5+, NS6 y Mozilla
  // Autor: Scott Andrew
  if ( elemento.addEventListener )
    {
    elemento.addEventListener (evType, funcion, useCapture);
    return true;
    }
  else if ( elemento.attachEvent )
    {
    var r = elemento.attachEvent ('on' + evType, funcion);
    return r;
    }
  else
    { elemento ['on' + evType] = funcion; }
  }
